/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


    /// <summary> Defines a single-valued association to another entity class that
    /// has many-to-one multiplicity. It is not normally necessary to
    /// specify the target entity explicitly since it can usually be
    /// inferred from the type of the object being referenced.  
    /// <para>
    /// If the
    /// relationship is bidirectional, the non-owning
    /// <c>OneToMany</c> entity side must used the
    /// <c>mappedBy</c> element to specify the relationship field or
    /// property of the entity that is the owner of the relationship.
    /// </para>
    /// <para>
    ///  The <c>ManyToOne</c> annotation may be used within an
    /// embeddable class to specify a relationship from the embeddable
    /// class to an entity class. If the relationship is bidirectional, the
    /// non-owning <c>OneToMany</c> entity side must use the <c>mappedBy</c>
    /// element of the <c>OneToMany</c> annotation to specify the
    /// relationship field or property of the embeddable field or property
    /// on the owning side of the relationship. The dot (".") notation
    /// syntax must be used in the <c>mappedBy</c> element to indicate the
    /// relationship attribute within the embedded attribute.  The value of
    /// each identifier used with the dot notation is the name of the
    /// respective embedded field or property.
    /// </para>
    /// <example><code>
    /// 
    ///     Example 1:
    /// 
    ///     &#91;ManyToOne(optional=false)&#93;
    ///     &#91;JoinColumn(name="CUST_ID", nullable=false, updatable=false)&#93;
    ///     public Customer getCustomer() { return customer; }
    /// 
    /// 
    ///     Example 2:
    /// 
    ///     &#91;Entity&#93;
    ///        public class Employee {
    ///        &#91;Id&#93; int id;
    ///        &#91;Embedded&#93; JobInfo jobInfo;
    ///        ...
    ///     }
    /// 
    ///     &#91;Embeddable&#93;
    ///        public class JobInfo {
    ///        String jobDescription; 
    ///        &#91;ManyToOne&#93; ProgramManager pm; // Bidirectional
    ///     }
    /// 
    ///     &#91;Entity&#93;
    ///        public class ProgramManager {
    ///        &#91;Id&#93; int id;
    ///        &#91;OneToMany(mappedBy="jobInfo.pm")&#93;
    ///        Collection&#060;Employee&#062; manages;
    ///     }
    /// 
    /// </code></example>
    /// 
    /// </summary>
    
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)] 


public sealed class ManyToOne : System.Attribute {

    /// <summary> 
    /// (Optional) The entity class that is the target of 
    /// the association. 
    /// 
    ///  Defaults to the type of the field or property 
    /// that stores the association. 
    /// </summary>
    public System.Type TargetEntity = typeof(void);

    /// <summary> (Optional) The operations that must be cascaded to 
    /// the target of the association.
    /// 
    ///  By default no operations are cascaded.
    /// </summary>
    public CascadeType[] Cascade = {};

    /// <summary> 
    /// (Optional) Whether the association should be lazily 
    /// loaded or must be eagerly fetched. The EAGER
    /// strategy is a requirement on the persistence provider runtime that 
    /// the associated entity must be eagerly fetched. The LAZY 
    /// strategy is a hint to the persistence provider runtime.
    /// </summary>
    public FetchType Fetch = FetchType.EAGER;

    /// <summary> 
    /// (Optional) Whether the association is optional. If set 
    /// to false then a non-null relationship must always exist.
    /// </summary>
    public bool Optional = true;
}
}