/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    /// <summary> Specifies the mapping for the key column of a map whose
    /// map key is a basic type. 
    /// <para>
    /// If the <c>name</c> element is not specified, it
    /// defaults to the concatenation of the following: the name of the
    /// referencing relationship field or property; "_"; "KEY".
    /// </para>
    /// <example><code>
    ///    Example:
    /// 
    ///    &#91;Entity&#93;
    ///    public class Item {
    ///       &#91;Id&#93; int id;
    ///       ...
    ///       &#91;ElementCollection&#93;
    ///       &#91;MapKeyColumn(name="IMAGE_NAME")&#93;
    ///       &#91;Column(name="IMAGE_FILENAME")&#93;
    ///       &#91;CollectionTable(name="IMAGE_MAPPING")&#93;
    ///       Dictionary&#060;String, String&#062; images;  // map from image name to filename
    ///       ...
    ///    } 
    /// </code></example>
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)]/*@Target( { METHOD, FIELD })*/

public sealed class MapKeyColumn : System.Attribute {

	/// <summary> (Optional) The name of the map key column. The table in which it is found
	/// depends upon the context. If the map key is for an element collection,
	/// the map key column is in the collection table for the map value. If the
	/// map key is for a ManyToMany entity relationship or for a OneToMany entity
	/// relationship using a join table, the map key column is in a join table.
	/// If the map key is for a OneToMany entity relationship using a foreign key
	/// mapping strategy, the map key column is in the table of the entity that
	/// is the value of the map.
        ///  Defaults to the concatenation of the following: the name of
        /// the referencing relationship field or property; "_"; "<c>KEY</c>".
	/// </summary>
	public string Name = "";

	/// <summary> (Optional) Whether the column is a unique key. This is a
	/// shortcut for the <c>UniqueConstraint</c> annotation
	/// at the table level and is useful for when the unique key
	/// constraint corresponds to only a single column. This
	/// constraint applies in addition to any constraint entailed
	/// by primary key mapping and to constraints specified at the
	/// table level.
	/// </summary>
	public bool Unique = false;

	/// <summary> (Optional) Whether the database column is nullable.
    /// </summary>
	public bool Nullable = false;

	/// <summary> (Optional) Whether the column is included in SQL INSERT statements
	/// generated by the persistence provider.
	/// </summary>
	public bool Insertable = true;

	/// <summary> (Optional) Whether the column is included in SQL UPDATE statements
	/// generated by the persistence provider.
	/// </summary>
	public bool Updatable = true;

	/// <summary> (Optional) The SQL fragment that is used when generating the DDL for the
	/// column.
	///  Defaults to the generated SQL to create a
	/// column of the inferred type.
        /// 
	/// </summary>
	public string ColumnDefinition = "";

	/// <summary> (Optional) The name of the table that contains the column. 
        /// 
        ///  Defaults: If the map key is for an element collection,
        /// the name of the collection table for the map value. If the
        /// map key is for a OneToMany or ManyToMany entity
        /// relationship using a join table, the name of the join table
        /// for the map. If the map key is for a OneToMany entity
        /// relationship using a foreign key mapping strategy, the name
        /// of the primary table of the entity that is the value of the
        /// map.
        /// </summary>
	public string Table = "";

	/// <summary> (Optional) The column length. (Applies only if a string-valued column is
	/// used.)
	/// </summary>
	public int Length = 255;

	/// <summary> (Optional) The precision for a decimal (exact numeric) column. (Applies
	/// only if a decimal column is used.)  
        /// 
        ///  Default: 0. (The value must be set by the developer.)
	/// </summary>
	public int Precision = 0; // decimal precision

	/// <summary> (Optional) The scale for a decimal (exact numeric) column. (Applies only
	/// if a decimal column is used.)
	/// </summary>
	public int Scale = 0; // decimal scale
}
}