/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    /// <summary> Specifies a primary key column that is used as a foreign key to
    /// join to another table.
    /// <para>
    ///  It is used to join the primary table of an entity subclass 
    /// in the <see cref="InheritanceType.JOINED"/> mapping strategy 
    /// to the primary table of its superclass; it is used within a 
    /// <see cref="SecondaryTable"/> annotation to join a secondary table 
    /// to a primary table; and it may be used in a <see cref="OneToOne"/> 
    /// mapping in which the primary key of the referencing entity 
    /// is used as a foreign key to the referenced entity. 
    /// </para><para>
    ///  If no <c>PrimaryKeyJoinColumn</c> annotation is 
    /// specified for a subclass in the <c>JOINED</c>
    /// mapping strategy, the foreign key columns are assumed 
    /// to have the same names as the primary key columns of the 
    /// primary table of the superclass.
    /// </para>
    /// <example><code>
    /// 
    ///    Example: Customer and ValuedCustomer subclass
    /// 
    ///    &#91;Entity&#93;
    ///    &#91;Table(name="CUST")&#93;
    ///    &#91;Inheritance(strategy=JOINED)&#93;
    ///    &#91;DiscriminatorValue("CUST")&#93;
    ///    public class Customer { ... }
    ///    
    ///    &#91;Entity&#93;
    ///    &#91;Table(name="VCUST")&#93;
    ///    &#91;DiscriminatorValue("VCUST")&#93;
    ///    &#91;PrimaryKeyJoinColumn(name="CUST_ID")&#93;
    ///    public class ValuedCustomer extends Customer { ... }
    /// </code>
    /// </example>
    /// 
    /// <seealso cref="SecondaryTable"/>
    /// <seealso cref="Inheritance"/>
    /// <seealso cref="OneToOne"/>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Class | System.AttributeTargets.Field | System.AttributeTargets.Property, AllowMultiple = true)]


    public sealed class PrimaryKeyJoinColumn : System.Attribute {

        /// <summary>
        /// (Optional) Only required when this PrimaryKeyJoinColumn belongs to a SecondaryTable annontaion.
        /// 
        /// If populated, this PrimaryKeyJoinColumn represetns a column that is used to join with 
        /// the primary table.
        ///  Defaults to the column(s) of the same name(s) 
        /// as the primary key column(s) in the primary table.
        /// </summary>
        public string tableRef = "";

        /// <summary> 
        /// (Optional) The name of the primary key column of the current table.
        ///  Defaults to the same name as the primary key column 
        /// of the primary table of the superclass (<c>JOINED</c> mapping strategy); the same 
        /// name as the primary key column of the primary table 
        /// (<c>SecondaryTable</c> mapping); or the same name as the 
        /// primary key column for the table for the referencing entity 
        /// (<c>OneToOne</c> mapping).
        /// </summary>
        public string Name = "";

        /// <summary> 
        /// (Optional) The name of the primary key column of the table
        /// being joined to.   Defaults to the same name as the primary
        /// key column of the primary table of the superclass
        /// (<c>JOINED</c> mapping strategy); the same name as the
        /// primary key column of the primary table
        /// (<c>SecondaryTable</c> mapping); or the same name as the
        /// primary key column for the table for the referencing entity
        /// (<c>OneToOne</c> mapping).
        /// </summary>
        public string ReferencedColumnName = "";

        /// <summary> (Optional) The SQL fragment that is used when generating the 
        /// DDL for the column. This should not be specified for a 
        /// <c>OneToOne</c> primary key association.
        ///  Defaults to the generated SQL to create a column of the 
        /// inferred type.
        /// </summary>
        public string ColumnDefinition = "";
    }
}