/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

/// <summary> Interface used to control query execution.
/// 
/// <seealso cref="T:TypedQuery`1"/>
/// <seealso cref="T:Parameter`1"/>
/// 
/// 
/// </summary>
public interface Query {

    /// <summary> Execute a SELECT query and return the query results
    /// as an untyped List.
    /// </summary>
    /// <returns> a list of the results</returns>
    /// <exception cref="InvalidOperationException"> if called for a Java
    ///         Persistence query language UPDATE or DELETE statement</exception>
    /// <exception cref="QueryTimeoutException"> if the query execution exceeds
    ///         the query timeout value set and only the statement is
    ///         rolled back</exception>
    /// <exception cref="TransactionRequiredException"> if a lock mode has
    ///         been set and there is no transaction</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking
    ///         fails and only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if the query execution exceeds 
    ///         the query timeout value set and the transaction 
    ///         is rolled back </exception>
    
    IList GetResultList();

    /// <summary> Execute a SELECT query that returns a single untyped result.</summary>
    /// <returns> the result</returns>
    /// <exception cref="NoResultException"> if there is no result</exception>
    /// <exception cref="NonUniqueResultException"> if more than one result</exception>
    /// <exception cref="InvalidOperationException"> if called for a Java
    ///         Persistence query language UPDATE or DELETE statement</exception>
    /// <exception cref="QueryTimeoutException"> if the query execution exceeds
    ///         the query timeout value set and only the statement is
    ///         rolled back</exception>
    /// <exception cref="TransactionRequiredException"> if a lock mode has
    ///         been set and there is no transaction</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking
    ///         fails and only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if the query execution exceeds 
    ///         the query timeout value set and the transaction 
    ///         is rolled back </exception> 
    /// 
    object GetSingleResult();

    /// <summary> Execute an update or delete statement.</summary>
    /// <returns> the number of entities updated or deleted</returns>
    /// <exception cref="InvalidOperationException"> if called for a Java
    ///         Persistence query language SELECT statement or for
    ///         a criteria query</exception>
    /// <exception cref="TransactionRequiredException"> if there is 
    ///         no transaction</exception>
    /// <exception cref="QueryTimeoutException"> if the statement execution 
    ///         exceeds the query timeout value set and only 
    ///         the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if the query execution exceeds 
    ///         the query timeout value set and the transaction 
    ///         is rolled back </exception> 
    /// 
    int ExecuteUpdate();

    /// <summary> Set the maximum number of results to retrieve.</summary>
    /// <param name="maxResult">  maximum number of results to retrieve</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the argument is negative</exception>
    /// 
    Query SetMaxResults(int maxResult);

    /// <summary> The maximum number of results the query object was set to
    /// retrieve. Returns <c>Integer.MAX_VALUE</c> if <c>setMaxResults</c> was not
    /// applied to the query object.</summary>
    /// <returns> maximum number of results</returns>
    /// 
    /// 
    int GetMaxResults();

    /// <summary> Set the position of the first result to retrieve.</summary>
    /// <param name="startPosition"> position of the first result, </param>
    /// numbered from 0
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the argument is negative</exception>
    /// 
    Query SetFirstResult(int startPosition);

    /// <summary> The position of the first result the query object was set to
    /// retrieve. Returns 0 if <c>setFirstResult</c> was not applied to the
    /// query object.</summary>
    /// <returns> position of the first result</returns>
    /// 
    /// 
    int GetFirstResult();

    /// <summary> Set a query property or hint. The hints elements may be used 
    /// to specify query properties and hints. Properties defined by
    /// this specification must be observed by the provider. 
    /// Vendor-specific hints that are not recognized by a provider
    /// must be silently ignored. Portable applications should not
    /// rely on the standard timeout hint. Depending on the database
    /// in use and the locking mechanisms used by the provider,
    /// this hint may or may not be observed.</summary>
    /// <param name="hintName">  name of the property or hint</param>
    /// <param name="value">  value for the property or hint</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the second argument is not
    ///         valid for the implementation</exception>
    /// 
    Query SetHint(string hintName, object value);

    /// <summary> Get the properties and hints and associated values that are 
    /// in effect for the query instance.</summary>
    /// <returns> query properties and hints</returns>
    /// 
    /// 
    Dictionary<string, object> GetHints();

    /// <summary> Bind the value of a <c>Parameter</c> object.</summary>
    /// <param name="param">  parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter
    ///         does not correspond to a parameter of the
    ///         query</exception>
    /// 
    /// 
    Query SetParameter<T> (Parameter<T> param, T value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a <c>Parameter</c> object.</summary>
    /// <param name="param"> parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter does not
    ///         correspond to a parameter of the query</exception>
    /// 
    /// 
    //Query SetParameter(Parameter<DateTime> param, DateTime value, 
    //                   TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a <c>Parameter</c> object.</summary>
    /// <param name="param"> parameter object</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter does not
    ///         correspond to a parameter of the query</exception>
    /// 
    /// 
    Query SetParameter(Parameter<System.DateTime> param, System.DateTime value, 
                       TemporalType temporalType);

    /// <summary> Bind an argument to a named parameter.</summary>
    /// <param name="name">  parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does 
    ///         not correspond to a parameter of the query or if
    ///         the argument is of incorrect type</exception>
    /// 
    Query SetParameter(string name, object value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a named parameter.</summary>
    /// <param name="name">  parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does 
    ///         not correspond to a parameter of the query or if
    ///         the value argument is of incorrect type</exception>
    /// 
    //Query SetParameter(string name, DateTime value, 
    //                   TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a named parameter.</summary>
    /// <param name="name">  parameter name</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if the parameter name does 
    ///         not correspond to a parameter of the query or if
    ///         the value argument is of incorrect type</exception>
    /// 
    Query SetParameter(string name, System.DateTime value, 
                       TemporalType temporalType);

    /// <summary> Bind an argument to a positional parameter.</summary>
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the
    ///         query or if the argument is of incorrect type</exception>
    /// 
    Query SetParameter(int position, object value);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a positional
    /// parameter.</summary>
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the query or
    ///         if the value argument is of incorrect type</exception>
    /// 
    //Query SetParameter(int position, DateTime value,  
    //                   TemporalType temporalType);

    /// <summary> Bind an instance of <c>System.DateTime</c> to a positional parameter.</summary>
    /// <param name="position">  position</param>
    /// <param name="value">  parameter value</param>
    /// <param name="temporalType">  temporal type</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="ArgumentException"> if position does not
    ///         correspond to a positional parameter of the query or
    ///         if the value argument is of incorrect type</exception>
    /// 
    Query SetParameter(int position, System.DateTime value,  
                       TemporalType temporalType);

    /// <summary> Get the parameter objects corresponding to the declared
    /// parameters of the query.
    /// Returns empty set if the query has no parameters.
    /// This method is not required to be supported for native
    /// queries.</summary>
    /// <returns> set of the parameter objects</returns>
    /// <exception cref="InvalidOperationException"> if invoked on a native
    ///         query when the implementation does not support 
    ///         this use</exception>
    /// 
    /// 
    HashSet<Object> GetParameters();

    /// <summary> Get the parameter object corresponding to the declared
    /// parameter of the given name.
    /// This method is not required to be supported for native
    /// queries.</summary>
    /// <param name="name">  parameter name</param>
    /// <returns> parameter object</returns>
    /// <exception cref="ArgumentException"> if the parameter of the
    ///         specified name does not exist</exception>
    /// <exception cref="InvalidOperationException"> if invoked on a native
    ///         query when the implementation does not support 
    ///         this use</exception>
    /// 
    /// 
    Parameter<T> GetParameter<T>(string name);

    /// <summary> Get the parameter object corresponding to the declared
    /// parameter of the given name and type.
    /// This method is required to be supported for criteria queries
    /// only.</summary>
    /// <param name="name">  parameter name</param>
    /// <param name="type">  type</param>
    /// <returns> parameter object</returns>
    /// <exception cref="ArgumentException"> if the parameter of the
    ///         specified name does not exist or is not assignable
    ///         to the type</exception>
    /// <exception cref="InvalidOperationException"> if invoked on a native
    ///         query or Java Persistence query language query when
    ///         the implementation does not support this use</exception>
    /// 
    /// 
    Parameter<T> GetParameter<T> (string name, System.Type type);

    /// <summary> Get the parameter object corresponding to the declared
    /// positional parameter with the given position.
    /// This method is not required to be supported for native
    /// queries.</summary>
    /// <param name="position">  position</param>
    /// <returns> parameter object</returns>
    /// <exception cref="ArgumentException"> if the parameter with the
    ///         specified position does not exist</exception>
    /// <exception cref="InvalidOperationException"> if invoked on a native
    ///         query when the implementation does not support 
    ///         this use</exception>
    /// 
    /// 
    Parameter<T> GetParameter<T>(int position);

    /// <summary> Get the parameter object corresponding to the declared
    /// positional parameter with the given position and type.
    /// This method is not required to be supported by the provider.</summary>
    /// <param name="position">  position</param>
    /// <param name="type">  type</param>
    /// <returns> parameter object</returns>
    /// <exception cref="ArgumentException"> if the parameter with the
    ///         specified position does not exist or is not assignable
    ///         to the type</exception>
    /// <exception cref="InvalidOperationException"> if invoked on a native
    ///         query or Java Persistence query language query when
    ///         the implementation does not support this use</exception>
    /// 
    /// 
    Parameter<T> GetParameter<T> (int position, System.Type type);

    /// <summary> Return a boolean indicating whether a value has been bound 
    /// to the parameter.</summary>
    /// <param name="param"> parameter object</param>
    /// <returns> boolean indicating whether parameter has been bound</returns>
    /// 
    /// 
    bool IsBound<T>(Parameter<T> param);

    /// <summary> Return the value bound to the parameter.</summary>
    /// <param name="param"> parameter object</param>
    /// <returns> parameter value</returns>
    /// <exception cref="ArgumentException"> if the parameter is not 
    ///         a parameter of the query</exception>
    /// <exception cref="InvalidOperationException"> if the parameter has not been
    ///         been bound</exception>
    /// 
    /// 
    T GetParameterValue<T> (Parameter<T> param);

    /// <summary> Return the value bound to the named parameter.</summary>
    /// <param name="name">  parameter name</param>
    /// <returns> parameter value</returns>
    /// <exception cref="InvalidOperationException"> if the parameter has not been
    ///         been bound</exception>
    /// <exception cref="ArgumentException"> if the parameter of the
    ///         specified name does not exist</exception>
    /// 
    /// 
    object GetParameterValue(string name);

    /// <summary> Return the value bound to the positional parameter.</summary>
    /// <param name="position">  position</param>
    /// <returns> parameter value</returns>
    /// <exception cref="InvalidOperationException"> if the parameter has not been
    ///         been bound</exception>
    /// <exception cref="ArgumentException"> if the parameter with the
    ///         specified position does not exist</exception>
    /// 
    /// 
    object GetParameterValue(int position);

    /// <summary> Set the flush mode type to be used for the query execution.
    /// The flush mode type applies to the query regardless of the
    /// flush mode type in use for the entity manager.</summary>
    /// <param name="flushMode">  flush mode</param>
    /// <returns> the same query instance</returns>
    /// 
    Query SetFlushMode(FlushModeType flushMode);

    /// <summary> Get the flush mode in effect for the query execution. 
    /// If a flush mode has not been set for the query object, 
    /// returns the flush mode in effect for the entity manager.</summary>
    /// <returns> flush mode</returns>
    /// 
    /// 
    FlushModeType GetFlushMode();

    /// <summary> Set the lock mode type to be used for the query execution.</summary>
    /// <param name="lockMode">  lock mode</param>
    /// <returns> the same query instance</returns>
    /// <exception cref="InvalidOperationException"> if the query is found not to be 
    ///         a Java Persistence query language SELECT query
    ///         or a Criteria API query</exception>
    /// 
    /// 
    Query SetLockMode(LockModeType lockMode);

    /// <summary> Get the current lock mode for the query.</summary>
    /// <returns> lock mode</returns>
    /// <exception cref="InvalidOperationException"> if the query is found not to be
    ///         a Java Persistence query language SELECT query or
    ///         a Criteria API query</exception>
    /// 
    /// 
    LockModeType GetLockMode();

    /// <summary> Return an object of the specified type to allow access to 
    /// the provider-specific API.  If the provider's query 
    /// implementation does not support the specified class, the 
    /// <c>PersistenceException</c> is thrown.</summary>
    /// <param name="cls">  the class of the object to be returned.  This is</param>
    ///             normally either the underlying query 
    ///             implementation class or an interface that it 
    ///             implements.
    /// <returns> an instance of the specified class</returns>
    /// <exception cref="PersistenceException"> if the provider does not support
    ///         the call</exception>
    /// 
    /// 
    T Unwrap<T> (System.Type cls);
}
}